#!/bin/bash

############################################################################################################
#
# Threshold Controller
# Autor: Aleritty <aleritty//#AT#//aleritty.net> http://www.aleritty.net
#	Questo script permette di tenere sotto controllo la propria connessione mobile
#	e permette di non oltrepassare le soglie del proprio operatore, evitando sorprese in bolletta.
#
#
#
#	esecuzione: ~$./connessione.sh [--help | --init | -i interf | --mon ]
#		Per uscire dal programma premere il tasto "q" se si vogliono salvare le soglie oppure CTRL+C
#		se non si desidera farlo.
#
#	Ricordarsi di dare allo script i ermessi di esecuzione con il comando:
#		~$chmod +x connessione.sh
#
#	Se si desidera lanciare il programma come comando di sistema può usare:
#		~#cp connessione.sh /usr/bin/connessione <-- eseguire come superutente
#	Da questo momento è possibile collegarsi in questa maniera:
#		~$connessione
#	Il comando sarà disponibile a tutti gli utenti del sistema, ma solo quelli autorizzati alla
#	connessione potranno realmente collegarsi.
#	Le statistiche verranno salvate nella corretta cartella per ogni utente.
#
#	Per motivi di sicurezza è necessario leggere le impostazioni per potersi connettere, quindi
#	decommentare la riga che effettua il test invece della connessione vera e propria.
#
#
#	Questo script è rilasciato sotto licenza creative commons: http://creativecommons.org/licenses/by-nc-sa/2.5/it/
#	E' quindi possibile: riprodurre, distribuire, comunicare al pubblico, esporre in pubblico, eseguire, modificare quest'opera
#	Alle seguenti condizioni:
#	Attribuzione. Devi attribuire la paternità dell'opera nei modi indicati dall'autore o da chi ti ha dato l'opera
#	in licenza e in modo tale da non suggerire che essi avallino te o il modo in cui tu usi l'opera.
#
#	Non commerciale. Non puoi usare quest'opera per fini commerciali.
#
#	Condividi allo stesso modo. Se alteri o trasformi quest'opera, o se la usi per crearne un'altra, puoi distribuire
#	l'opera risultante solo con una licenza identica o equivalente a questa.
#
#	Questo script agisce sui file contenuti nella cartella indicata dall'utente, pertanto l'autore non si ritiene responsabile
#	di qualsiasi danno o perdita di dati derivata dall'uso improprio o inconsapevole di questo script!
#	Le uniche soglie ufficiali sono quelle fornite dal vostro operatore telefonico, l'autore non è responsabile di eventuali addebiti
#	anche in caso di conteggio errato del programma.
#
#
#	to-do: ancora molto... (implementazione statistiche, implementazione kill pid fornito a mano)

# CONFIGURAZIONI DA IMPOSTARE!!

interf="ppp0" 				#indicare l'interfaccia da monitorare, normalmente ppp0

soglia="99MB" 				#indicare il numero di byte al quale costringere la disconnessione! indicare sempre l'unità di misura (B,KB,MB,GB) 1KB=1024B

tipo_soglia="G"				#indicare il tipo di soglia alla quale ci si riferisce (G,S,MS,ML)

percorso_file_config="$HOME/.TC" 	#cartella che contiene il file di configurazione (SENZA slash finale)
file_config="stats" 			#nome del file contenente le statistiche

t_refresh="5" 				#Ogni quanti secondi vengono aggiornati i dati

#test="ok" 				#Decommentare la riga se si desidera testare il programma senza effettuare realmente una connessione, viene eseguito un ping su loopback per incrementare i dati trasmessi!

#monitor_mode="ok" 			#impostare ad 1 se si usa il programma solo per monitorare la connessione, ma ci si connette con un'altro programma.


################################## non toccare al di sotto di questa linea!! ########################
Version="0.4"

################################## inizializzazione #################################################

if [[ "$1" == "--init" || ! -e "$percorso_file_config/$file_config" ]]
then
	if [ -e "$percorso_file_config/$file_config" ]
	then
		echo -n "il file di configurazione esiste, vuoi sovrascriverlo? [y/n]:"
		read pressione
		if [[ "$pressione" != "y" ]]
		then
			echo "ok, NON lo sovrascrivo!"
			exit
		fi
	fi
	echo "creazione dei files di configurazione"
	echo "le configurazioni verranno create in: $percorso_file_config/$file_config"
	mkdir -p $percorso_file_config
	touch "$percorso_file_config/$file_config"
	echo "dati totali scambiati    dati ricevuti    dati trasmessi    data inizio connessione    data fine connessione   interfaccia" > "$percorso_file_config/$file_config"
	exit
fi

################################## fine inizializzazione #############################################

################################## Help ##############################################################

if [[ "$1" == "--help" ]]
then
	echo "Threshold Controller $Version
Scritto da: AleRitty <aleritty#\@/#aleritty.net> http://www.aleritty.net
Mantiene sotto controllo le soglie imposte dal proprio operatore di telefonia mobile

Uso:
~$bash connessione.sh [ --help | --init | -i interf | --mon ]

--init 			Inizializza o resetta le configurazioni dello script
-i interf		Specifica l'interfaccia monitorata SOLO per la sessione corrente
--mon			Abilita la modalità monitor
--help			Mostra questa schermata
"
fi

################################## fine help #########################################################

################################## Interfaccia manuale ###############################################

if [[ "$1" == "-i" ]]
then
	interf=$2
fi

################################## fine Interfaccia manuale ##########################################

################################## Monitor mode ######################################################

if [[ "$1" == "--mon" ]]
then
	monitor_mode="ok"
fi

################################## fine Monitor mode #################################################

echo -n "[sudo] password for $USER: "
stty -echo
read password
stty echo
echo

################################## determinazione valore soglia ######################################
suffisso=${soglia:(( ${#soglia}-2 )):${#soglia}}

case $suffisso in
	"GB")
	val_soglia=${soglia%"GB"}
	let "val_soglia=$val_soglia*1073741824"
	;;
#se soglia termina con MB allora moltiplico per 1048576
	"MB")
	val_soglia=${soglia%"MB"}
	let "val_soglia=$val_soglia*1048576"
	;;
#se soglia termina con KB allora moltiplico per 1024
	"KB")
	val_soglia=${soglia%"KB"}
	let "val_soglia=$val_soglia*1024"
	;;
#se soglia termina con un numero oppure con B non (quasi) faccio nulla
	*)
	if [[ ${soglia:(( ${#soglia}-1 )):${#soglia}} == "B" ]]
	then val_soglia=${soglia%"B"}
	else val_soglia=$soglia
	fi
	;;
esac

#########################Determinazione tempo della soglia #######################

case $tipo_soglia in
#se la soglia è giornaliera
	"G")
	limite_temporale="86399"
	;;
	"S")
	limite_temporale="604799"
	;;
#	"MS")
#	limite_temporale="Un po' più complicato"
#	;;
	"ML")
	limite_temporale="18143999"
	;;
esac


##lettura dati precedenti nel file di configurazione

leggo_dati=`cat "$percorso_file_config/$file_config" | grep "$interf" | tail -n1`
arr_dati=($leggo_dati)
##prendo in considerazione i dati precedenti solo se l'apertura della sessione precedente è stata nello stesso limite temporale!
dapertura=`date +%s`
if [[ "$monitor_mode" != "ok" ]]
then
	let "delta_temp=$dapertura-${arr_dati[3]}"
	if (( "$delta_temp" < "$limite_temporale" ))
	then
		echo "Vediamo quanti dati hai già consumato per questa soglia"
		dati_letti=${arr_dati[0]}
		let "MB_letti=${arr_dati[0]}/1048576"
		let "KB_letti=${arr_dati[0]}/1024"
		echo "dati consumati: $dati_letti B ($MB_letti MB, $KB_letti KB)"
	else
		echo "Non hai ancora consumato dati, inizio a conteggiare da 0"
		dati_letti=0
	fi
else
	dati_letti=0
fi

##se oggi ho già superato la soglia allora...
if (( $dati_letti<$val_soglia ))
then
	auth_conn="ok"
else
	echo -n "Attenzione: hai già superato la soglia, vuoi continuare lo stesso? [y/n]:"
	read pressione
	if [[ "$pressione" != "y" ]]
	then
		echo "Bravo!"
		exit
	else
		echo "Ok, come preferisci..."
		auth_conn="ok"
	fi
fi


#####qui effettuo la connessione se sono stato autorizzato!
if [[ "$auth_conn" == "ok" && "$monitor_mode" != "ok" ]]
then
	if [[ "$test" == "ok" ]]
	then
		ping localhost &> /dev/null &
	else
		echo $password | sudo -S wvdial&
		sleep 5
	fi
	pid_connect=$!
fi

##entrata in loop per ascoltare i dati!
echo
while [ TRUE ]
do
	##monitoraggio ogni X secondi della soglia, con stampa a terminale
	riga=`cat /proc/net/dev | grep $interf | cut -d ":" -f2`
#	riga=`cat /proc/net/dev | grep $interf | sed -e "s/$interf://"` #Versione vecchia, usa più memoria, tenuta per motivi di compatibilita'
	dato=($riga)
	dati_ricevuti=${dato[0]}
	dati_trasmessi=${dato[8]}
	let "dati_totali=$dati_ricevuti+$dati_trasmessi+$dati_letti"
	let "mb_scambiati=$dati_totali / 1048576"
	let "kb_scambiati=$dati_totali / 1024"
	echo -ne "$interf RX:$dati_ricevuti TX:$dati_trasmessi-->$dati_totali ($mb_scambiati MB, $kb_scambiati KB)\r"



##se il valore si avvicina alla soglia impostata: interruzione della sessione!
	let "limite_soglia=$val_soglia*0,99"
	if (( $dati_totali>$limite_soglia )) && [[ "$auth_conn" == "ok" ]]
	then
		echo "Disconnessione per raggiungimento soglia di $val_soglia"
		zenity --warning --text "ATTENZIONE! \n \
Hai quasi superato la soglia di internet" --title "Treshold Controller" --timeout 30
		break
	fi

	read -t $t_refresh -n1 pressioni
	if [[ "$pressioni" == "q" ]]
	then
		echo
		echo "Disconnessione..."
		break
	fi
done


##su chiusura sessione, scrivere nel file la soglia raggiunta, con la data e l'ora di INIZIO sessione ed un po' di dati per le statistiche (da implementare)!
echo $password | sudo -S kill $pid_connect
echo "$dati_totali $dati_ricevuti $dati_trasmessi $dapertura `date +%s` $interf" >> "$percorso_file_config/$file_config"
echo "Sessione di navigazione chiusa, dati salvati correttamente, a presto"
echo
